@@ -12,12 +12,8 @@
 /* fix compilation issue on SunOS 5.10, see:
  * https://github.com/giampaolo/psutil/issues/421
  * https://github.com/giampaolo/psutil/issues/1077
- * http://us-east.manta.joyent.com/jmc/public/opensolaris/ARChive/PSARC/2010/111/materials/s10ceval.txt
- *
- * Because LEGACY_MIB_SIZE defined in the same file there is no way to make autoconfiguration =\
 */
 
-#define NEW_MIB_COMPLIANT 1
 #define _STRUCTURED_PROC 1
 
 #include <Python.h>
@@ -44,6 +40,14 @@
 #include <sys/tihdr.h>
 #include <stropts.h>
 #include <inet/tcp.h>
+#ifndef NEW_MIB_COMPLIANT
+/*
+ * Solaris introduced NEW_MIB_COMPLIANT macro with Update 4.
+ * See https://github.com/giampaolo/psutil/issues/421
+ * Prior to Update 4, one has to include mib2 by hand.
+ */
+#include <inet/mib2.h>
+#endif
 #include <arpa/inet.h>
 #include <net/if.h>
 #include <math.h> // fabs()
@@ -271,7 +275,7 @@ psutil_proc_environ(PyObject *self, PyObject *args) {
         goto error;
 
     if (! info.pr_envp) {
-        AccessDenied("");
+        AccessDenied("/proc/pid/psinfo struct not set");
         goto error;
     }
 
@@ -300,8 +304,8 @@ psutil_proc_environ(PyObject *self, PyObject *args) {
         if (PyDict_SetItem(py_retdict, py_envname, py_envval) < 0)
             goto error;
 
-        Py_DECREF(py_envname);
-        Py_DECREF(py_envval);
+        Py_CLEAR(py_envname);
+        Py_CLEAR(py_envval);
     }
 
     psutil_free_cstrings_array(env, env_count);
@@ -492,7 +496,7 @@ proc_io_counters(PyObject* self, PyObject* args) {
                          info.pr_inblk,
                          info.pr_oublk);
 }
- */
+*/
 
 
 /*
@@ -655,10 +659,10 @@ psutil_users(PyObject *self, PyObject *args) {
             goto error;
         if (PyList_Append(py_retlist, py_tuple))
             goto error;
-        Py_DECREF(py_username);
-        Py_DECREF(py_tty);
-        Py_DECREF(py_hostname);
-        Py_DECREF(py_tuple);
+        Py_CLEAR(py_username);
+        Py_CLEAR(py_tty);
+        Py_CLEAR(py_hostname);
+        Py_CLEAR(py_tuple);
     }
     endutxent();
 
@@ -714,9 +718,9 @@ psutil_disk_partitions(PyObject *self, PyObject *args) {
             goto error;
         if (PyList_Append(py_retlist, py_tuple))
             goto error;
-        Py_DECREF(py_dev);
-        Py_DECREF(py_mountp);
-        Py_DECREF(py_tuple);
+        Py_CLEAR(py_dev);
+        Py_CLEAR(py_mountp);
+        Py_CLEAR(py_tuple);
     }
     fclose(file);
     return py_retlist;
@@ -767,8 +771,7 @@ psutil_per_cpu_times(PyObject *self, PyObject *args) {
                 goto error;
             if (PyList_Append(py_retlist, py_cputime))
                 goto error;
-            Py_DECREF(py_cputime);
-            py_cputime = NULL;
+            Py_CLEAR(py_cputime);
         }
     }
 
@@ -824,7 +827,7 @@ psutil_disk_io_counters(PyObject *self, PyObject *args) {
                 if (PyDict_SetItemString(py_retdict, ksp->ks_name,
                                          py_disk_info))
                     goto error;
-                Py_DECREF(py_disk_info);
+                Py_CLEAR(py_disk_info);
             }
         }
         ksp = ksp->ks_next;
@@ -959,8 +962,8 @@ psutil_proc_memory_maps(PyObject *self, PyObject *args) {
             goto error;
         if (PyList_Append(py_retlist, py_tuple))
             goto error;
-        Py_DECREF(py_path);
-        Py_DECREF(py_tuple);
+        Py_CLEAR(py_path);
+        Py_CLEAR(py_tuple);
 
         // increment pointer
         p += 1;
@@ -1021,7 +1024,7 @@ psutil_net_io_counters(PyObject *self, PyObject *args) {
             goto next;
 
         // check if this is a network interface by sending a ioctl
-        strncpy(ifr.lifr_name, ksp->ks_name, sizeof(ifr.lifr_name));
+        PSUTIL_STRNCPY(ifr.lifr_name, ksp->ks_name, sizeof(ifr.lifr_name));
         ret = ioctl(sock, SIOCGLIFFLAGS, &ifr);
         if (ret == -1)
             goto next;
@@ -1075,7 +1078,7 @@ psutil_net_io_counters(PyObject *self, PyObject *args) {
             goto error;
         if (PyDict_SetItemString(py_retdict, ksp->ks_name, py_ifc_info))
             goto error;
-        Py_DECREF(py_ifc_info);
+        Py_CLEAR(py_ifc_info);
         goto next;
 
 next:
@@ -1120,7 +1123,7 @@ psutil_net_connections(PyObject *self, PyObject *args) {
     mib2_udp6Entry_t     ude6;
 #endif
     char buf[512];
-    int i, flags, getcode, num_ent, state;
+    int i, flags, getcode, num_ent, state, ret;
     char lip[INET6_ADDRSTRLEN], rip[INET6_ADDRSTRLEN];
     int lport, rport;
     int processed_pid;
@@ -1147,7 +1150,7 @@ psutil_net_connections(PyObject *self, PyObject *args) {
         goto error;
     }
 
-    int ret = ioctl(sd, I_PUSH, "tcp");
+    ret = ioctl(sd, I_PUSH, "tcp");
     if (ret == -1) {
         PyErr_SetFromErrno(PyExc_OSError);
         goto error;
@@ -1273,7 +1276,7 @@ psutil_net_connections(PyObject *self, PyObject *args) {
                     goto error;
                 if (PyList_Append(py_retlist, py_tuple))
                     goto error;
-                Py_DECREF(py_tuple);
+                Py_CLEAR(py_tuple);
             }
         }
 #if defined(AF_INET6)
@@ -1287,7 +1290,7 @@ psutil_net_connections(PyObject *self, PyObject *args) {
 #ifdef NEW_MIB_COMPLIANT
                 processed_pid = tp6.tcp6ConnCreationProcess;
 #else
-        		processed_pid = 0;
+                        processed_pid = 0;
 #endif
                 if (pid != -1 && processed_pid != pid)
                     continue;
@@ -1316,14 +1319,14 @@ psutil_net_connections(PyObject *self, PyObject *args) {
                     goto error;
                 if (PyList_Append(py_retlist, py_tuple))
                     goto error;
-                Py_DECREF(py_tuple);
+                Py_CLEAR(py_tuple);
             }
         }
 #endif
         // UDPv4
         else if (mibhdr.level == MIB2_UDP || mibhdr.level == MIB2_UDP_ENTRY) {
             num_ent = mibhdr.len / sizeof(mib2_udpEntry_t);
-	    assert(num_ent * sizeof(mib2_udpEntry_t) == mibhdr.len);
+            assert(num_ent * sizeof(mib2_udpEntry_t) == mibhdr.len);
             for (i = 0; i < num_ent; i++) {
                 memcpy(&ude, databuf.buf + i * sizeof ude, sizeof ude);
 #ifdef NEW_MIB_COMPLIANT
@@ -1355,7 +1358,7 @@ psutil_net_connections(PyObject *self, PyObject *args) {
                     goto error;
                 if (PyList_Append(py_retlist, py_tuple))
                     goto error;
-                Py_DECREF(py_tuple);
+                Py_CLEAR(py_tuple);
             }
         }
 #if defined(AF_INET6)
@@ -1388,7 +1391,7 @@ psutil_net_connections(PyObject *self, PyObject *args) {
                     goto error;
                 if (PyList_Append(py_retlist, py_tuple))
                     goto error;
-                Py_DECREF(py_tuple);
+                Py_CLEAR(py_tuple);
             }
         }
 #endif
@@ -1434,10 +1437,10 @@ psutil_boot_time(PyObject *self, PyObject *args) {
 
 
 /*
- * Return the number of physical CPU cores on the system.
+ * Return the number of CPU cores on the system.
  */
 static PyObject *
-psutil_cpu_count_phys(PyObject *self, PyObject *args) {
+psutil_cpu_count_cores(PyObject *self, PyObject *args) {
     kstat_ctl_t *kc;
     kstat_t *ksp;
     int ncpus = 0;
@@ -1512,7 +1515,7 @@ psutil_net_if_stats(PyObject* self, PyObject* args) {
             if (strcmp(ksp->ks_class, "net") != 0)
                 continue;
 
-            strncpy(ifr.lifr_name, ksp->ks_name, sizeof(ifr.lifr_name));
+            PSUTIL_STRNCPY(ifr.lifr_name, ksp->ks_name, sizeof(ifr.lifr_name));
             ret = ioctl(sock, SIOCGLIFFLAGS, &ifr);
             if (ret == -1)
                 continue;  // not a network interface
@@ -1561,7 +1564,7 @@ psutil_net_if_stats(PyObject* self, PyObject* args) {
                 goto error;
             if (PyDict_SetItemString(py_retdict, ksp->ks_name, py_ifc_info))
                 goto error;
-            Py_DECREF(py_ifc_info);
+            Py_CLEAR(py_ifc_info);
         }
     }
 
@@ -1632,52 +1635,31 @@ error:
 static PyMethodDef
 PsutilMethods[] = {
     // --- process-related functions
-    {"proc_basic_info", psutil_proc_basic_info, METH_VARARGS,
-     "Return process ppid, rss, vms, ctime, nice, nthreads, status and tty"},
-    {"proc_name_and_args", psutil_proc_name_and_args, METH_VARARGS,
-     "Return process name and args."},
-    {"proc_environ", psutil_proc_environ, METH_VARARGS,
-      "Return process environment."},
-    {"proc_cpu_times", psutil_proc_cpu_times, METH_VARARGS,
-     "Return process user and system CPU times."},
-    {"proc_cred", psutil_proc_cred, METH_VARARGS,
-     "Return process uids/gids."},
-    {"query_process_thread", psutil_proc_query_thread, METH_VARARGS,
-     "Return info about a process thread"},
-    {"proc_memory_maps", psutil_proc_memory_maps, METH_VARARGS,
-     "Return process memory mappings"},
-    {"proc_num_ctx_switches", psutil_proc_num_ctx_switches, METH_VARARGS,
-     "Return the number of context switches performed by process"},
-    {"proc_cpu_num", psutil_proc_cpu_num, METH_VARARGS,
-     "Return what CPU the process is on"},
+    {"proc_basic_info", psutil_proc_basic_info, METH_VARARGS},
+    {"proc_cpu_num", psutil_proc_cpu_num, METH_VARARGS},
+    {"proc_cpu_times", psutil_proc_cpu_times, METH_VARARGS},
+    {"proc_cred", psutil_proc_cred, METH_VARARGS},
+    {"proc_environ", psutil_proc_environ, METH_VARARGS},
+    {"proc_memory_maps", psutil_proc_memory_maps, METH_VARARGS},
+    {"proc_name_and_args", psutil_proc_name_and_args, METH_VARARGS},
+    {"proc_num_ctx_switches", psutil_proc_num_ctx_switches, METH_VARARGS},
+    {"query_process_thread", psutil_proc_query_thread, METH_VARARGS},
 
     // --- system-related functions
-    {"swap_mem", psutil_swap_mem, METH_VARARGS,
-     "Return information about system swap memory."},
-    {"users", psutil_users, METH_VARARGS,
-     "Return currently connected users."},
-    {"disk_partitions", psutil_disk_partitions, METH_VARARGS,
-     "Return disk partitions."},
-    {"per_cpu_times", psutil_per_cpu_times, METH_VARARGS,
-     "Return system per-CPU times."},
-    {"disk_io_counters", psutil_disk_io_counters, METH_VARARGS,
-     "Return a Python dict of tuples for disk I/O statistics."},
-    {"net_io_counters", psutil_net_io_counters, METH_VARARGS,
-     "Return a Python dict of tuples for network I/O statistics."},
-    {"boot_time", psutil_boot_time, METH_VARARGS,
-     "Return system boot time in seconds since the EPOCH."},
-    {"cpu_count_phys", psutil_cpu_count_phys, METH_VARARGS,
-     "Return the number of physical CPUs on the system."},
-    {"net_connections", psutil_net_connections, METH_VARARGS,
-     "Return TCP and UDP syste-wide open connections."},
-    {"net_if_stats", psutil_net_if_stats, METH_VARARGS,
-     "Return NIC stats (isup, duplex, speed, mtu)"},
-    {"cpu_stats", psutil_cpu_stats, METH_VARARGS,
-     "Return CPU statistics"},
+    {"boot_time", psutil_boot_time, METH_VARARGS},
+    {"cpu_count_cores", psutil_cpu_count_cores, METH_VARARGS},
+    {"cpu_stats", psutil_cpu_stats, METH_VARARGS},
+    {"disk_io_counters", psutil_disk_io_counters, METH_VARARGS},
+    {"disk_partitions", psutil_disk_partitions, METH_VARARGS},
+    {"net_connections", psutil_net_connections, METH_VARARGS},
+    {"net_if_stats", psutil_net_if_stats, METH_VARARGS},
+    {"net_io_counters", psutil_net_io_counters, METH_VARARGS},
+    {"per_cpu_times", psutil_per_cpu_times, METH_VARARGS},
+    {"swap_mem", psutil_swap_mem, METH_VARARGS},
+    {"users", psutil_users, METH_VARARGS},
 
     // --- others
-    {"set_testing", psutil_set_testing, METH_NOARGS,
-     "Set psutil in testing mode"},
+    {"set_debug", psutil_set_debug, METH_VARARGS},
 
     {NULL, NULL, 0, NULL}
 };
@@ -1748,7 +1730,14 @@ void init_psutil_sunos(void)
     PyModule_AddIntConstant(module, "SSTOP", SSTOP);
     PyModule_AddIntConstant(module, "SIDL", SIDL);
     PyModule_AddIntConstant(module, "SONPROC", SONPROC);
+#ifdef SWAIT
     PyModule_AddIntConstant(module, "SWAIT", SWAIT);
+#else
+    /* sys/proc.h started defining SWAIT somewhere
+     * after Update 3 and prior to Update 5 included.
+     */
+    PyModule_AddIntConstant(module, "SWAIT", 0);
+#endif
 
     PyModule_AddIntConstant(module, "PRNODEV", PRNODEV);  // for process tty
 
